@@ -185,8 +185,8 @@ def compare_derivatives(func, numerical_derivatives,
                         # This message is useful: the user can see that
                         # tests are really performed (instead of not being
                         # performed, silently):
-                        print "Testing %s at %s, arg #%d" % (
-                            funcname, args, arg_num)
+                        print "Testing derivative #%d of %s at %s" % (
+                            arg_num, funcname, args_scalar)
 
                         if not numbers_close(fixed_deriv_value,
                                              num_deriv_value, 1e-4):
@@ -2132,7 +2132,8 @@ else:
         both their nominal value and uncertainty are equal (up to the
         given precision).
 
-        m1, m2 -- NumPy matrices.
+        m1, m2 -- NumPy arrays.
+
         precision -- precision passed through to
         uncertainties.test_uncertainties.numbers_close().
         """
@@ -2155,11 +2156,12 @@ else:
             if not numbers_close(elmt1.std_dev,
                                  elmt2.std_dev, precision):
                 return False
+        
         return True
 
 
     def test_numpy_comparison():
-        "Comparison with a Numpy array."
+        "Comparison with a NumPy array."
 
         x = ufloat(1, 0.1)
 
@@ -2277,6 +2279,44 @@ else:
         assert numbers_close(corr_matrix[0,0], 1)
         assert numbers_close(corr_matrix[1,2], 2*v.std_dev/sum_value.std_dev)
 
+        ####################
+
+        # Test of numerical robustness despite wildly different
+        # orders of magnitude (see
+        # https://github.com/lebigot/uncertainties/issues/95):
+        cov = numpy.diag([1e-70, 1e-70, 1e10])
+        cov[0, 1] = cov[1, 0] = 0.9e-70
+        cov[[0, 1], 2] = -3e-34
+        cov[2, [0, 1]] = -3e-34
+        variables = uncert_core.correlated_values([0]*3, cov)
+
+        # Since the numbers are very small, we need to compare them
+        # in a stricter way, that handles the case of a 0 variance
+        # in `variables`:
+        assert numbers_close(
+                1e66*cov[0,0], 1e66*variables[0].s**2, tolerance=1e-5)
+        assert numbers_close(
+                1e66*cov[1,1], 1e66*variables[1].s**2, tolerance=1e-5)
+
+        ####################
+
+        # 0 variances are a bit special, since the correlation matrix
+        # cannot be calculated naively, so we test that there is no
+        # specific problem in this case:
+
+        cov = numpy.diag([0, 0, 10])
+        nom_values = [1, 2, 3]
+        variables = uncert_core.correlated_values(nom_values, cov)
+
+        for (variable, nom_value, variance) in zip(
+            variables, nom_values, cov.diagonal()):
+            
+            assert numbers_close(variable.n, nom_value)
+            assert numbers_close(variable.s**2, variance) 
+        
+        assert arrays_close(
+            cov,
+            numpy.array(uncert_core.covariance_matrix(variables)))
 
     def test_correlated_values_correlation_mat():
         '''
